; #####################################

; SHOW_ROTATE, SHOW_TX, SHOW_RX, MSG (and WART), SHOW_STATS, SHOW_PROTO:
; functions used to output different kinds of data to the screen
; (or possibly an offscreen buffer, config is in terminal-data.asm)

SHOW_ROTATE:	; rotates position of status-, proto- and help-line
	push bx
	mov bx, word [activeline]
	inc bx
	cmp bx, updatedflags_size
	jb .gotnext
	xor bx, bx
.gotnext:
	mov word [activeline], bx
	mov byte [updatedflags + bx], -1
	pop bx
	; jmp update_proto_stat_msg

update_proto_stat_msg:
	db __TEST_IMM8			; (NC)
.update:
	stc
	push bx
	push si
	push di
	push cx
	push ax

	mov bx, word [activeline]
	jc @F
	rol byte [updatedflags + bx], 1
	jnc .done
@@:
	mov byte [updatedflags + bx], 0

	mov ax, 160
	mov cx, ax
	mul bl
	mov si, ax
	add si, buffers
	mov bx, word [helpline]
	xchg bl, bh
	BX2DI
	rep movsb

.done:
	pop ax
	pop cx
	pop di
	pop si
	pop bx
	retn


; #####################################

SHOW_TX:        ; show AL/AX in mini-circular-rawbuffer:
		; the local echo facility!
        PUSH ax
        PUSH bx
        PUSH di
	push es
	 push cs
	 pop es
        cmp ax,-1
        jz wasnt_sent
        mov bx,ax
        mov di,[cs:tpos]
_txok:  mov ax,[cs:attr]
        mov al,bl
	rol byte [cs:option_cpdump], 1
	jc @F
	cmp al, 32
	jb .dot
	cmp al, 127
	jb @F
.dot:
	mov al, '.'
@@:
        stosw
        mov [tpos],di
        cmp di,[cs:tend]
        jb _txok2
        mov di,[cs:tstart]
        mov [tpos],di
_txok2: mov ax,[cs:attr]
        stosw
	mov byte [protoupdated], -1
wasnt_sent:
	pop es
        POP di
        POP bx
        POP ax
	jmp update_proto_stat_msg


; #####################################

SHOW_RX:        ; show char in AL/AX in main output window
	cmp byte [ans_expect],1	; flag: we are inside an ANSI sequence
	jz srx_ansi
	cmp al,27	; char 27 (ESC) would start a new sequence
	jz srx_new_ansi

srx_tty:	; we are outside ANSI mode	
                call TTY
	jmp short srx_done

srx_new_ansi:
	mov byte [ans_expect],1	; now we are inside a new sequence
	mov word [abufptr],abuffer	; *offset* flush recovery buffer
	mov word [aparptr],apars	; *offset* flush argument buffer
	xor ax,ax
	mov [apars],ax			; flush argument buffer
	mov [apars+2],ax
	mov [abuffer],ax		; really flush recovery buffer
	jmp short srx_done	; the ESC is implicit and not processed

srx_ansi:	; we are really inside a sequence
		call ANSI
		nop
		nop
		nop
	jnc srx_done	; sequence still in progress
	jz srx_recover	; sequence was aborted
srx_eof:	; carry: end of sequence or error in sequence
	mov byte [ans_expect],0		; flag: no longer inside sequence
	jmp short srx_done

srx_recover:	; sequence could not be parsed
	push bx
	mov bx,arecover	; *offset*
srx_recshow:		; show the buffered unparsed sequence
	mov al,[cs:bx]
	inc bx
	test al, al
	jz srx_recdone
		call TTY
	jmp short srx_recshow	; loop until 0 encountered
srx_recdone:	; done with recovering
	mov al,[cs:bx-2]	; last non NULL char
	cmp al,27
	jnz srx_notweird	; previous sequence aborted because of ESC?
	mov byte [ans_expect],0
	call SHOW_RX		; we are NESTING to store the fact!
	pop bx
	jmp short srx_done	; do not reset ans_expect: new seq has begun
srx_notweird:
	pop bx
	jmp short srx_eof	; sequence is aborted -and- at end

srx_done:
        RET

; #####################################

MSG:		; to show a help message or similar stuff:
        PUSH ax	; show string on DS:SI to special message area on screen,
        PUSH si	; special char 1 means "wait and scroll"
        PUSH di
	push es
	 push cs
	 pop es
        mov di, msgbuffer
        mov ax,[cs:attr]
msglp:  lodsb
        dec al
        jz msglf
        inc al
        jz msgeof
mnorm:  stosw
        cmp di,[cs:msgend]
        jb msglp
        mov di,[cs:msgstart]
        jmp short msglp
msglf:
	pop es
	push es
	PUSH cx
        mov cx,60
	rol byte [skipintro], 1
	jc .skipwarten
	mov byte [msgupdated], -1
	call update_proto_stat_msg
                call WART
.skipwarten:
	 push cs
	 pop es
        mov di,[cs:msgstart]
        mov ax,[cs:attr]
        mov cx,[cs:cols]
        rep stosw
        POP cx
        mov di,[cs:msgstart]
        jmp short msglp
msgeof:
	pop es
	POP di
        POP si
        POP ax
	mov byte [msgupdated], -1
	jmp update_proto_stat_msg

; helper function for MSG to wait for a while:

WART:   PUSH ax         ; CX V-Syncs (8, 1 waere H-Syncs)
        PUSH dx
        mov dx,[cs:crtport]
        add dx,6
wmono:  in al,dx
        IODLY     ; 6x86 delay
        test al,8
        jz wmono
wmono2: in al,dx
        IODLY     ; 6x86 delay
        test al,8
        jnz wmono2
        loop wmono
        POP dx
        POP ax
        RET

; #####################################

SHOW_STATS:	; converts the 16bit comstats value to a human-readable
	PUSH ax	; list of keywords
        PUSH bx
        PUSH cx
        PUSH dx
        PUSH si
        PUSH di
	push es
	mov ax,[cs:comstats]
	 push cs
	 pop es
	mov di, statbuffer
        mov si,mybits	; *offset*
        mov cx,16       ; 16 Bits
        mov bx,ax
statlp: mov ax,[cs:attr]
        mov al,ah
        shl bx,1
        jc stat1
stat0:  and ah,077h     ; no brigth/blink
stat1:  mov dx,5        ; fixed len
statsh: lodsb
        stosw
        dec dx
        jnz statsh
        loop statlp
	mov byte [statupdated], -1
	pop es
        POP di
        POP si
        POP dx
        POP cx
        POP bx
        POP ax
	jmp update_proto_stat_msg

; #####################################

SHOW_PROTO:	; shows line speed, CR policies for send and receive,
        PUSH ax	; and [the low byte hshake as a bit field]
        PUSH bx	; new 10/2001: show ANSI [simulated LED bit field] as well
        PUSH cx
        PUSH si
        PUSH di
	push es
	 push cs
	 pop es
	mov di, protobuffer
pr_sp:  mov si,[cs:linespeed]
        shl si,1
        shl si,1
        lea si,[bauds+si]	; select one of the speed descriptions
        mov ax,[cs:attr]
        mov cx,4
pr_lp1: lodsb
        stosw
        loop pr_lp1
        mov al,' '
        stosw
        mov si,[cs:crpolicy]
        shl si,1
        shl si,1
        lea si,[crsends+si]	; select one of the CR send descriptions
        mov cx,4
pr_lp2: lodsb
        stosw
        loop pr_lp2
        mov al,' '
        stosw
        mov bx,[cs:hshake]
        and bx,00ffh
        test word [crpolicy],8000h
        jz pr_na
        or bx,100h	; convert CR receive policy to a "9th hshake bit"
pr_na:  mov cl,7
        shl bx,cl
        mov cx,9
        mov si,crgets	; *offset*	; has to be right in front of hshakes!
pr_lp3: mov ax,[cs:attr]	; convert the hshake bit field to a string
        shl bx,1
        jc pr_on
pr_off: and ah,77h      ; no bright/blink
pr_on:  PUSH cx
        mov cx,5
pr_lp4: lodsb
        stosw
        loop pr_lp4
        POP cx
        loop pr_lp3
	mov al,' '
	stosw
	stosw
pr_led:	mov bl,[cs:rled]	; prepare display of LED simulation
	mov si,rleds	; *offset*
	mov cx,4
pr_lp5:	mov ax,[cs:attr]
	test bl,1
	jnz pr_le1
	and ah,11h	; off means darker... and blue-er :-)
pr_le1:	lodsb
	stosw
	shr bl,1	; next LED (low bit shown first...)
	loop pr_lp5
	mov al,' '
	stosw
	stosw
	mov byte [protoupdated], -1
	pop es
        POP di
        POP si
        POP cx
        POP bx
        POP ax
	jmp update_proto_stat_msg


bx2di:
        xchg di,ax	; save AX
        mov ax,[cs:cols]	; screen width
        mul bh		; use row
        add al,bl	; add column
        adc ah,0
        shl ax,1
        xchg di,ax	; restore AX: no registers are destroyed!
	retn

; #####################################
